package cn.com.infosec.crypto.engines;

import cn.com.infosec.crypto.BlockCipher;
import cn.com.infosec.crypto.CipherParameters;
import cn.com.infosec.crypto.InvalidCipherTextException;
import cn.com.infosec.crypto.Wrapper;
import cn.com.infosec.crypto.modes.CBCBlockCipher;
import cn.com.infosec.crypto.params.ParametersWithIV;
import cn.com.infosec.crypto.params.ParametersWithRandom;
import java.security.SecureRandom;

public class RFC3211WrapEngine implements Wrapper {
   private CBCBlockCipher engine;
   private ParametersWithIV param;
   private boolean forWrapping;
   private SecureRandom rand;

   public RFC3211WrapEngine(BlockCipher engine) {
      RFC3211WrapEngine var10000 = this;
      super();
      CBCBlockCipher var2;
      var2 = new CBCBlockCipher.<init>(engine);
      var10000.engine = var2;
   }

   public void init(boolean forWrapping, CipherParameters param) {
      this.forWrapping = forWrapping;
      if (param instanceof ParametersWithRandom) {
         RFC3211WrapEngine var10000 = this;
         ParametersWithRandom var3;
         this.rand = (var3 = (ParametersWithRandom)param).getRandom();
         var10000.param = (ParametersWithIV)var3.getParameters();
      } else {
         if (forWrapping) {
            SecureRandom forWrapping1;
            forWrapping1 = new SecureRandom.<init>();
            this.rand = forWrapping1;
         }

         this.param = (ParametersWithIV)param;
      }

   }

   public String getAlgorithmName() {
      return this.engine.getUnderlyingCipher().getAlgorithmName() + "/RFC3211Wrap";
   }

   public byte[] wrap(byte[] in, int inOff, int inLen) {
      if (!this.forWrapping) {
         throw new IllegalStateException("not set for wrapping");
      } else {
         this.engine.init(true, this.param);
         int var4 = this.engine.getBlockSize();
         int var5;
         int var6;
         byte[] var8;
         if ((var5 = inLen + 4) < (var6 = var4 * 2)) {
            var8 = new byte[var6];
         } else {
            if (var5 % var4 == 0) {
               var6 = var5;
            } else {
               var6 = (var5 / var4 + 1) * var4;
            }

            var8 = new byte[var6];
         }

         var8[0] = (byte)inLen;
         var8[1] = (byte)(~in[inOff]);
         var8[2] = (byte)(~in[inOff + 1]);
         var8[3] = (byte)(~in[inOff + 2]);
         System.arraycopy(in, inOff, var8, 4, inLen);

         while(var5 < var8.length) {
            var8[var5] = (byte)this.rand.nextInt();
            ++var5;
         }

         int in1;
         for(in1 = 0; in1 < var8.length; in1 += var4) {
            this.engine.processBlock(var8, in1, var8, in1);
         }

         for(in1 = 0; in1 < var8.length; in1 += var4) {
            this.engine.processBlock(var8, in1, var8, in1);
         }

         return var8;
      }
   }

   public byte[] unwrap(byte[] in, int inOff, int inLen) {
      if (this.forWrapping) {
         throw new IllegalStateException("not set for unwrapping");
      } else {
         int var4;
         if (inLen < (var4 = this.engine.getBlockSize()) * 2) {
            throw new InvalidCipherTextException("input too short");
         } else {
            byte[] var10001 = in;
            int var10002 = inOff;
            byte[] var10003 = in;
            int var10004 = inOff;
            in = new byte[inLen];
            byte[] inOff1 = new byte[var4];
            System.arraycopy(var10003, var10004, in, 0, inLen);
            System.arraycopy(var10001, var10002, inOff1, 0, var4);
            CBCBlockCipher var10000 = this.engine;
            ParametersWithIV var5;
            var5 = new ParametersWithIV.<init>(this.param.getParameters(), inOff1);
            var10000.init(false, var5);

            for(int var9 = var4; var9 < inLen; var9 += var4) {
               this.engine.processBlock(in, var9, in, var9);
            }

            System.arraycopy(in, inLen - var4, inOff1, 0, var4);
            CBCBlockCipher var10 = this.engine;
            var5 = new ParametersWithIV.<init>(this.param.getParameters(), inOff1);
            var10.init(false, var5);
            this.engine.processBlock(in, 0, in, 0);
            this.engine.init(false, this.param);

            for(inOff = 0; inOff < inLen; inOff += var4) {
               this.engine.processBlock(in, inOff, in, inOff);
            }

            if ((in[0] & 255) > inLen - 4) {
               throw new InvalidCipherTextException("wrapped key corrupted");
            } else {
               byte[] var6 = new byte[in[0] & 255];
               byte inOff2 = in[0];
               System.arraycopy(in, 4, var6, 0, inOff2);

               for(inOff = 0; inOff != 3; inOff = inLen) {
                  if ((byte)(~in[inLen = inOff + 1]) != var6[inOff]) {
                     throw new InvalidCipherTextException("wrapped key fails checksum");
                  }
               }

               return var6;
            }
         }
      }
   }
}
